package systems.archide.core.pojo;

import java.util.ArrayList;
import java.util.List;

import com.google.common.base.CaseFormat;

import systems.archide.core.controller.search.Fuzziness;

public class QueryBuilder {
    private List<Object> parameters = new ArrayList<>();

    public PreparedQuery buildQuery(SearchQueryInput input, String tableName, Long limit, Long offset, String orderBy) {
        StringBuilder query = new StringBuilder("SELECT * FROM " + tableName + " WHERE ");
        for (int i = 0; i < input.getColumns().size(); i++) {
        	 if (i < input.getColumns().size() - 1)
        		 appendCondition(query, input.getColumns().get(i), true, input.getCaseSensitiveSearch());
    		 else
    			 appendCondition(query, input.getColumns().get(i), false, input.getCaseSensitiveSearch());
        }
        if (orderBy != null)
			query.append(" order by " + orderBy);
		if (limit != null) 
			query.append(" limit " + limit);
		if (offset != null) {
			query.append(" offset " + offset);
		}
        return new PreparedQuery(query.toString(), parameters);
    }

    private void appendCondition(StringBuilder query, SearchColumn column, Boolean and, Boolean caseSensitiveSearch) {
        switch (column.getType().toLowerCase()) {
            case "text":
                handleTextType(query, column, and, caseSensitiveSearch);
                break;
            case "number":
                handleNumberType(query, column, and);
                break;
            case "boolean":
                handleBooleanType(query, column, and);
                break;
            case "date":
                handleDateType(query, column, and);
                break;
            case "enum":
                handleEnumType(query, column, and);
                break;
            default:
                throw new IllegalArgumentException("Unsupported column type: " + column.getType());
        }
    }

    private void handleTextType(StringBuilder query, SearchColumn column, Boolean and, Boolean caseSensitiveSearch) {
    	if (column.getTextValue() != null) {
	        if (column.getSearchType().equals("fuzzy")) {
	            query.append("(SIMILARITY(").append(formatColumnName(column.getName())).append(", ?) > ").append(Fuzziness.AUTO.getFuzzinessValue()).append(") ");
	            parameters.add(column.getTextValue());
	        } else if (column.getSearchType().equals("exact")) {
	            query.append(formatColumnName(column.getName())).append(" = ?");
	            parameters.add(column.getTextValue());
	        } else if (column.getSearchType().equals("startsWith")) {
                if (caseSensitiveSearch) {
	                query.append(formatColumnName(column.getName())).append(" LIKE ?");
                } else {
	                query.append(formatColumnName(column.getName())).append(" ILIKE ?");
                }
	            parameters.add(column.getTextValue() + "%");
	        } else if (column.getSearchType().equals("notStartsWith")) {
                if (caseSensitiveSearch) {
	                query.append(formatColumnName(column.getName())).append(" NOT LIKE ?");
                } else {
                    query.append(formatColumnName(column.getName())).append(" NOT ILIKE ?");
                }
	            parameters.add(column.getTextValue() + "%");
	        } else {
	            query.append("(SIMILARITY(").append(formatColumnName(column.getName())).append(", ?) > ").append(Fuzziness.AUTO.getFuzzinessValue()).append(") ");
            	parameters.add(column.getTextValue());
	        }
	        if (and)
	        	query.append(" ").append("and").append(" ");
        	else
        		query.append(" ");
    	}
    }

    private void handleNumberType(StringBuilder query, SearchColumn column, Boolean and) {
        if (column.getLongRange() != null) {
            query.append(formatColumnName(column.getName())).append(" BETWEEN ? AND ?");
            parameters.add(column.getLongRange().getMin());
            parameters.add(column.getLongRange().getMax());
            if (and)
	        	query.append(" ").append("and").append(" ");
        	else
        		query.append(" ");
        } else if (column.getLongValue() != null) {
            query.append(formatColumnName(column.getName())).append(" = ?");
            parameters.add(column.getLongValue()); // Assuming value is provided
            if (and)
	        	query.append(" ").append("and").append(" ");
        	else
        		query.append(" ");
        }
    }
    
    private void handleDateType(StringBuilder query, SearchColumn column, Boolean and) {
        if (column.getDateRange() != null) {
            query.append(formatColumnName(column.getName())).append(" BETWEEN ? AND ?");
            parameters.add(column.getDateRange().getMin());
            parameters.add(column.getDateRange().getMax());
            if (and)
	        	query.append(" ").append("and").append(" ");
        	else
        		query.append(" ");
        } else if (column.getDateValue() != null) {
            query.append(formatColumnName(column.getName())).append(" = ?");
            parameters.add(column.getDateValue()); // Assuming value is provided
            if (and)
	        	query.append(" ").append("and").append(" ");
        	else
        		query.append(" ");
        }
    }

    private void handleBooleanType(StringBuilder query, SearchColumn column, Boolean and) {
    	if (column.getBooleanValue() != null) {
    		query.append(formatColumnName(column.getName())).append(" = ?");
            parameters.add(column.getBooleanValue());
            if (and)
            	query.append(" ").append("and").append(" ");
        	else
        		query.append(" ");
    	}
    }
    
    //TODO: This is hardcoded enum type (need to add enum type: shor or integer ...) 
    private void handleEnumType(StringBuilder query, SearchColumn column, Boolean and) {
    	if (column.getEnumValues() != null)
    		if (!column.getEnumValues().isEmpty()) {
    			query.append(formatColumnName(column.getName())).append(" IN (");
    	        for (int i = 0; i < column.getEnumValues().size(); i++) {
    	            query.append("?");
    	            if (i < column.getEnumValues().size() - 1) {
    	                query.append(", ");
    	            }
    	            parameters.add(Short.parseShort(column.getEnumValues().get(i)));
    	        }
    	        query.append(")");
    	        if (and)
    	        	query.append(" ").append("and").append(" ");
    	    	else
    	    		query.append(" ");
    		}
    }
    
    private String formatColumnName(String columnName) {
    	return CaseFormat.UPPER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, columnName);
    }

	public PreparedQuery buildQuery(SearchQueryInput input, String tableName) {
		if (!input.getColumns().isEmpty()) {
			StringBuilder query = new StringBuilder("select count(id) from " + tableName + " WHERE ");
	        for (int i = 0; i < input.getColumns().size(); i++) {
	        	 if (i < input.getColumns().size() - 1)
	        		 appendCondition(query, input.getColumns().get(i), true, input.getCaseSensitiveSearch());
	    		 else
	    			 appendCondition(query, input.getColumns().get(i), false, input.getCaseSensitiveSearch());
	        }
	        return new PreparedQuery(query.toString(), parameters);
		}
		return null;
	}

}



