package systems.archide.core.workflows;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.core.JsonProcessingException;

import systems.archide.core.ErrorCodes;
import systems.archide.core.pojo.Address;
import systems.archide.core.pojo.AddressType;
import systems.archide.core.pojo.ArchideMtlbAnnex;
import systems.archide.core.pojo.ArchideMtlbAnnexLogEntry;
import systems.archide.core.pojo.ArchideMtlbAnnexTag;
import systems.archide.core.pojo.ArchideMtlbAnnexUtil;
import systems.archide.core.pojo.CivilStatusMtlb;
import systems.archide.core.pojo.CivilStatusMtlbAttachment;
import systems.archide.core.pojo.CivilStatusMtlbNote;
import systems.archide.core.pojo.CivilStatusMtlbTag;
import systems.archide.core.pojo.Email;
import systems.archide.core.pojo.Face;
import systems.archide.core.pojo.GeoData;
import systems.archide.core.pojo.GlobalState;
import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.MtlbType;
import systems.archide.core.pojo.Occupation;
import systems.archide.core.pojo.OccupationMetadata;
import systems.archide.core.pojo.PhoneNumber;
import systems.archide.core.pojo.RvfourBirthMtlb;
import systems.archide.core.pojo.RvfourBirthMtlbAttachment;
import systems.archide.core.pojo.RvfourBirthMtlbTag;
import systems.archide.core.pojo.TypeOfAcquisition;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.pojo.VitalRecordAttachment;
import systems.archide.core.pojo.VitalRecordHistory;
import systems.archide.core.pojo.VitalRecordMention;
import systems.archide.core.pojo.VitalRecordNote;
import systems.archide.core.pojo.VitalRecordTag;
import systems.archide.core.repository.AddressRepository;
import systems.archide.core.repository.CivilStatusMtlbAttachmentRepository;
import systems.archide.core.repository.CivilStatusMtlbNoteRepository;
import systems.archide.core.repository.CivilStatusMtlbRepository;
import systems.archide.core.repository.CivilStatusMtlbTagRepository;
import systems.archide.core.repository.EmailRepository;
import systems.archide.core.repository.FaceRepository;
import systems.archide.core.repository.GeoDataRepository;
import systems.archide.core.repository.GlobalStateRepository;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.OccupationMetadataRepository;
import systems.archide.core.repository.OccupationRepository;
import systems.archide.core.repository.PhoneNumberRepository;
import systems.archide.core.repository.RvfourBirthMtlbAttachmentRepository;
import systems.archide.core.repository.RvfourBirthMtlbRepository;
import systems.archide.core.repository.RvfourBirthMtlbTagRepository;
import systems.archide.core.repository.VitalRecordAttachmentRepository;
import systems.archide.core.repository.VitalRecordHistoryRepository;
import systems.archide.core.repository.VitalRecordMentionRepository;
import systems.archide.core.repository.VitalRecordNoteRepository;
import systems.archide.core.repository.VitalRecordRepository;
import systems.archide.core.repository.VitalRecordTagRepository;
import systems.archide.core.workflows.utils.CustomErrorCodes;
import systems.archide.core.workflows.utils.RvFourBirthMtlbUtils;
import systems.archide.core.workflows.utils.Utils;

@Component
public class RvFourBirthMtlbSubmitted {

	private static final Logger log = LoggerFactory.getLogger(RvFourBirthMtlbSubmitted.class);

	
	final static String RELIGION_TAG = "Religion";

	final static String INVALID_BIRTH_PLACE_DATA = "RV4_ERR_1";
	final static String INVALID_RESIDENCE_ADDRESS_DATA = "RV4_ERR_2";

	@Autowired
	VitalRecordRepository vitalRecordRepository;

	@Autowired
	FaceRepository faceRepository;

	@Autowired
	VitalRecordTagRepository vitalRecordTagRepository;

	@Autowired
	VitalRecordAttachmentRepository vitalRecordAttachmentRepository;

	@Autowired
	AddressRepository addressRepository;

	@Autowired
	OccupationMetadataRepository occupationMetadataRepository;

	@Autowired
	EmailRepository emailRepository;

	@Autowired
	PhoneNumberRepository phoneNumberRepository;

	@Autowired
	OccupationRepository occupationRepository;

	@Autowired
	GeoDataRepository locationsRepository;

	@Autowired
	RvfourBirthMtlbRepository rvfourBirthMtlbRepository;

	@Autowired
	RvfourBirthMtlbTagRepository rvfourBirthMtlbTagRepository;

	@Autowired
	GlobalStateRepository globalStateRepository;

	@Autowired
	VitalRecordNoteRepository vitalRecordNoteRepository;

	@Autowired
	VitalRecordMentionRepository vitalRecordMentionRepository;

	@Autowired
	RvfourBirthMtlbAttachmentRepository rv4BirthMtlbAttachmentRepository;
	
	@Autowired
	RvFourBirthMtlbUtils rvFourBirthMtlbUtils;
	
	@Autowired
	VitalRecordHistoryRepository vitalRecordHistoryRepository;
	
	@Autowired
	CivilStatusMtlbRepository civilStatusMtlbRepository;
	
	@Autowired
	CivilStatusMtlbTagRepository civilStatusMtlbTagRepository;
	
	@Autowired
	CivilStatusMtlbAttachmentRepository civilStatusMtlbAttachmentRepository;
	
	@Autowired
	CivilStatusMtlbNoteRepository civilStatusMtlbNoteRepository;
	
	@Autowired
	IdcoreCustomQueries queries;
	
	@Autowired
	Utils utils;

	@Scheduled(fixedDelay = 5000)
	public void fromSubmittedStatus() {

		GlobalState migrationState = globalStateRepository.findByKey("migration");
		List<RvfourBirthMtlb> mtlbList = queries.getRandomRv4BirthMtlbByMtlbStatus(MtlbStatus.SUBMITTED.getValue());

		if (mtlbList.size() > 0) {
			log.info("Read {} newly submitted rv4 birth applications", mtlbList.size());
			for (RvfourBirthMtlb mtlb : mtlbList) {
				if (mtlb.getVitalRecordId() != null) {
					switch (mtlb.getMtlbType()) {
						case RV4_BIRTH_VITAL_RECORD:
							createVitalRecord(mtlb, migrationState);
							break;
						case RV4_VITAL_RECORD_HISTORY:
							createHistoryRecord(mtlb);
							break;
						case RV4_BIRTH_APPLICATION:
							createBirthApplication(mtlb, migrationState);
							break;
						default:
							mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
					}
					rvfourBirthMtlbRepository.save(mtlb);
				} else {
					mtlb.setMtlbStatus(MtlbStatus.MANUAL_CHECK);
					RvfourBirthMtlbTag tag = new RvfourBirthMtlbTag();
					tag.setRvfourBirthMtlbId(mtlb.getId());
					tag.setContent(ErrorCodes.VITAL_RECORD_ID_MISSING);
					tag.setType(Utils.ERROR_TAG_TYPE);
					rvfourBirthMtlbTagRepository.save(tag);
					rvfourBirthMtlbRepository.save(mtlb);
				}
			}
		}
	}

	private void createBirthApplication(RvfourBirthMtlb mtlb, GlobalState migrationState) {
		CivilStatusMtlb csMtlb = new CivilStatusMtlb();
		csMtlb.setMtlbStatus(MtlbStatus.SUBMITTED);
		csMtlb.setMtlbType(MtlbType.BIRTH_REGISTRATION);
		ArrayList<CivilStatusMtlbTag> tags = new ArrayList<CivilStatusMtlbTag>();
		if (mtlb.getFatherId() != null) {
			int fatherRecordStatus = utils.checkVitalRecordStatus(mtlb.getFatherId(), migrationState);
			if (fatherRecordStatus == Utils.FOUND)
				csMtlb.setFatherId(mtlb.getFatherId());
			else if (fatherRecordStatus == Utils.UNKNOWN || fatherRecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
				return;
			else if (fatherRecordStatus == Utils.CONFIRMED_MISSING) {
				CivilStatusMtlbTag tag = null;
				tag = new CivilStatusMtlbTag();
				tag.setContent(ErrorCodes.FATHER_NOT_FOUND);
				tag.setType(Utils.ERROR_TAG_TYPE);
				tags.add(tag);
			} else if (fatherRecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
				CivilStatusMtlbTag tag = null;
				tag = new CivilStatusMtlbTag();
				tag.setKey(CustomErrorCodes.RV4_FATHER_FAMILY_TREE_RECORD);
				tag.setContent(mtlb.getFatherId().toString());
				tag.setType(Utils.ERROR_TAG_TYPE);
				tags.add(tag);
			}
		}

		if (mtlb.getMotherId() != null) {
			int motherRecordStatus = utils.checkVitalRecordStatus(mtlb.getMotherId(), migrationState);
			if (motherRecordStatus == Utils.FOUND)
				csMtlb.setMotherId(mtlb.getMotherId());
			else if (motherRecordStatus == Utils.UNKNOWN)
				return;
			else if (motherRecordStatus == Utils.CONFIRMED_MISSING) {
				CivilStatusMtlbTag tag = null;
				tag = new CivilStatusMtlbTag();
				tag.setContent(ErrorCodes.MOTHER_NOT_FOUND);
				tag.setType(Utils.ERROR_TAG_TYPE);
				tags.add(tag);
			} else if (motherRecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
				CivilStatusMtlbTag tag = null;
				tag = new CivilStatusMtlbTag();
				tag.setKey(CustomErrorCodes.RV4_MOTHER_FAMILY_TREE_RECORD);
				tag.setContent(mtlb.getMotherId().toString());
				tag.setType(Utils.ERROR_TAG_TYPE);
				tags.add(tag);
			}
		}
		csMtlb.setFirstname(mtlb.getFirstname());
		csMtlb.setSecondname(mtlb.getSecondname());
		csMtlb.setThirdname(mtlb.getThirdname());
		csMtlb.setFourthname(mtlb.getFourthname());
		csMtlb.setFifthname(mtlb.getFifthname());
		csMtlb.setReportedFatherName(mtlb.getReportedFatherName());
		csMtlb.setReportedMotherName(mtlb.getReportedMotherName());
		csMtlb.setPrimaryCitizenship("191");
		csMtlb.setTypeOfAcquisition(TypeOfAcquisition.BY_DESCENT);
		rvFourBirthMtlbUtils.validateBiographic(mtlb);
		if (!StringUtils.isAllBlank(mtlb.getBirthPlace())) {
			GeoData location = locationsRepository.findByAreaId(mtlb.getBirthPlace());
			if (location != null)
				csMtlb.setBirthPlace(location.getAreaId());
			else {
				CivilStatusMtlbTag tag = new CivilStatusMtlbTag();
				tag.setContent(mtlb.getBirthPlace());
				tag.setType(Utils.WARNING_TAG_TYPE);
				tag.setKey(INVALID_BIRTH_PLACE_DATA);
				tags.add(tag);
			}
		}

		csMtlb.setBirthdate(mtlb.getBirthdate());
		csMtlb.setBirthYearUnknown(mtlb.isBirthYearUnknown());
		csMtlb.setBirthMonthUnknown(mtlb.isBirthMonthUnknown());
		csMtlb.setBirthDayUnknown(mtlb.isBirthDayUnknown());
		if (mtlb.getGender() != null)
			csMtlb.setGender(mtlb.getGender().shortValue());
		csMtlb.setMaritalStatus(mtlb.getMaritalStatus());

		Occupation occupation = null;
		if (!StringUtils.isAllBlank(mtlb.getOccupation())) {
			String legacyOccupation = mtlb.getOccupation().trim();
			occupation = new Occupation();
			List<OccupationMetadata> omList = occupationMetadataRepository.findByName(legacyOccupation);
			if (omList.size() > 0)
				occupation.setOccupationId(omList.get(0).getId());
			else {
				OccupationMetadata om = new OccupationMetadata();
				om.setName(legacyOccupation);
				occupationMetadataRepository.insert(om);
				occupation.setOccupationId(om.getId());
			}
		}

		Email email = null;
		if (!StringUtils.isAllBlank(mtlb.getEmail())) {
			String legacyEmail = mtlb.getEmail().trim();
			email = new Email();
			email.setEmail(legacyEmail);
		}

		PhoneNumber phoneNumber = null;
		if (!StringUtils.isAllBlank(mtlb.getPhoneNumber())) {
			String legacyPhoneNumber = mtlb.getPhoneNumber().trim();
			phoneNumber = new PhoneNumber();
			phoneNumber.setPhoneNumber(legacyPhoneNumber);
		}

		Address address = null;
		if (!StringUtils.isAllBlank(mtlb.getAddress())) {
			CivilStatusMtlbTag tag = null;
			GeoData location =locationsRepository.findByAreaId(mtlb.getAddress());
			if (location != null) {
				address = new Address();
				address.setAddressType(AddressType.RESIDENCY);
				address.setAreaId(location.getAreaId());
			}
			else {
				tag = new CivilStatusMtlbTag();
				tag.setContent(mtlb.getBirthPlace());
				tag.setType(Utils.WARNING_TAG_TYPE);
				tag.setKey(INVALID_RESIDENCE_ADDRESS_DATA);
				tags.add(tag);
			}
			if (tag != null)
				tags.add(tag);
		}

		if (mtlb.isMultiBirth()) {
			CivilStatusMtlbTag tag = new CivilStatusMtlbTag();
			tag.setContent("Multiple Births");
			tag.setType(Utils.INFO_TAG_TYPE);
			tags.add(tag);
		}

		if (mtlb.getFace() != null) {
			csMtlb.setFace(mtlb.getFace());
		}

		ArchideMtlbAnnex annex = null;
		try {
			annex = ArchideMtlbAnnexUtil.deserialize(mtlb.getAnnex());
		} catch (JsonProcessingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if (annex.getTags() != null)
			for (ArchideMtlbAnnexTag tag : annex.getTags()) {
				CivilStatusMtlbTag vt = new CivilStatusMtlbTag();
				vt.setContent(tag.getValue());
				vt.setType(tag.getType());
				vt.setKey(tag.getName());
				tags.add(vt);
			}
		
		
		List<RvfourBirthMtlbTag> mtlbTagList = rvfourBirthMtlbTagRepository.findByRvfourBirthMtlbId(mtlb.getId());
		for (RvfourBirthMtlbTag tag : mtlbTagList) {
			CivilStatusMtlbTag vt = new CivilStatusMtlbTag();
			vt.setContent(tag.getContent());
			vt.setType(tag.getType());
			vt.setKey(tag.getKey());
			tags.add(vt);
		}
		ArrayList<CivilStatusMtlbNote> notes = new ArrayList<>();
		for (ArchideMtlbAnnexLogEntry note : annex.getNotes()) {
			CivilStatusMtlbNote vn = new CivilStatusMtlbNote();
			vn.setAddedBy(note.getAuthor());
			vn.setContent(note.getContent());
			vn.setTimestamp(note.getTimestamp());
			notes.add(vn);
		}

		ArrayList<CivilStatusMtlbAttachment> attachments = new ArrayList<>();
		List<RvfourBirthMtlbAttachment> mtlbAttachments = rv4BirthMtlbAttachmentRepository
				.findByRvfourBirthMtlbId(mtlb.getId());
		for (RvfourBirthMtlbAttachment mtlbAttachment: mtlbAttachments) {
			CivilStatusMtlbAttachment attachment = new CivilStatusMtlbAttachment();
			attachment.setContent(mtlbAttachment.getContent());
			attachment.setFileName(mtlbAttachment.getFileName());
			attachment.setDescription(mtlbAttachment.getDescription());
			attachment.setPreview(mtlbAttachment.getPreview());
			attachments.add(attachment);
		}

		try {
			Long id  = civilStatusMtlbRepository.insert(csMtlb);
			if (occupation != null) {
				occupation.setCivilStatusMtlbId(id);
				occupationRepository.insert(occupation);
			}
			if (email != null) {
				email.setCivilStatusMtlbId(id);
				emailRepository.insert(email);
			}
			if (phoneNumber != null) {
				phoneNumber.setCivilStatusMtlbId(id);
				phoneNumberRepository.insert(phoneNumber);
			}
			if (address != null) {
				address.setCivilStatusMtlbId(id);
				addressRepository.insert(address);
			}
			for (CivilStatusMtlbTag tag : tags) {
				tag.setCivilStatusMtlbId(id);
				civilStatusMtlbTagRepository.insert(tag);
			}
			for (CivilStatusMtlbAttachment attachment : attachments) {
				attachment.setCivilStatusMtlbId(id);
				civilStatusMtlbAttachmentRepository.insert(attachment);
			}
			for (CivilStatusMtlbNote note : notes) {
				note.setCivilStatusMtlbId(id);
				civilStatusMtlbNoteRepository.insert(note);
			}
			mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}

	private void createHistoryRecord(RvfourBirthMtlb mtlb) {
		VitalRecordHistory vr = new VitalRecordHistory();
		vr.setId(mtlb.getVitalRecordId());
		vr.setFirstname(mtlb.getFirstname());
		vr.setSecondname(mtlb.getSecondname());
		vr.setThirdname(mtlb.getThirdname());
		vr.setFourthname(mtlb.getFourthname());
		vr.setFifthname(mtlb.getFifthname());
		vr.setReportedFatherName(mtlb.getReportedFatherName());
		vr.setReportedMotherName(mtlb.getReportedMotherName());
		vr.setFatherId(mtlb.getFatherId());
		vr.setMotherId(mtlb.getMotherId());
		if (!StringUtils.isAllBlank(mtlb.getBirthPlace())) {
			GeoData location = locationsRepository.findByAreaId(mtlb.getBirthPlace());
			if (location != null)
				vr.setBirthPlace(location.getAreaId());
		}
		vr.setTimestamp(mtlb.getTimestamp());
		vr.setBirthdate(mtlb.getBirthdate());
		vr.setBirthYearUnknown(mtlb.isBirthYearUnknown());
		vr.setBirthMonthUnknown(mtlb.isBirthMonthUnknown());
		vr.setBirthDayUnknown(mtlb.isBirthDayUnknown());
		vr.setPrimaryCitizenship("191");
		vr.setTypeOfAcquisition(TypeOfAcquisition.BY_DESCENT);
		if (mtlb.getGender() != null)
			vr.setGender(mtlb.getGender().shortValue());
		vr.setMaritalStatus(mtlb.getMaritalStatus());
		vr.setHistoryTimestamp(mtlb.getTimestamp());
		vitalRecordHistoryRepository.insert(vr);
		mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
	}

	private void createVitalRecord(RvfourBirthMtlb mtlb, GlobalState migrationState) {
		VitalRecord vr = new VitalRecord();
		vr.setInactive(false);
		ArrayList<VitalRecordTag> tags = new ArrayList<VitalRecordTag>();
		if (mtlb.getFatherId() != null) {
			int fatherRecordStatus = utils.checkVitalRecordStatus(mtlb.getFatherId(), migrationState);
			if (fatherRecordStatus == Utils.FOUND)
				vr.setFatherId(mtlb.getFatherId());
			else if (fatherRecordStatus == Utils.UNKNOWN || fatherRecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
				return;
			else if (fatherRecordStatus == Utils.CONFIRMED_MISSING) {
				VitalRecordTag tag = null;
				tag = new VitalRecordTag();
				tag.setVitalRecordId(mtlb.getVitalRecordId());
				tag.setContent(ErrorCodes.FATHER_NOT_FOUND);
				tag.setType(Utils.ERROR_TAG_TYPE);
				tags.add(tag);
			} else if (fatherRecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
				VitalRecordTag tag = null;
				tag = new VitalRecordTag();
				tag.setVitalRecordId(mtlb.getVitalRecordId());
				tag.setKey(CustomErrorCodes.RV4_FATHER_FAMILY_TREE_RECORD);
				tag.setContent(mtlb.getFatherId().toString());
				tag.setType(Utils.ERROR_TAG_TYPE);
				tags.add(tag);
			}
		}

		if (mtlb.getMotherId() != null) {
			int motherRecordStatus = utils.checkVitalRecordStatus(mtlb.getMotherId(), migrationState);
			if (motherRecordStatus == Utils.FOUND)
				vr.setMotherId(mtlb.getMotherId());
			else if (motherRecordStatus == Utils.UNKNOWN)
				return;
			else if (motherRecordStatus == Utils.CONFIRMED_MISSING) {
				VitalRecordTag tag = null;
				tag = new VitalRecordTag();
				tag.setVitalRecordId(mtlb.getVitalRecordId());
				tag.setContent(ErrorCodes.MOTHER_NOT_FOUND);
				tag.setType(Utils.ERROR_TAG_TYPE);
				tags.add(tag);
			} else if (motherRecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
				VitalRecordTag tag = null;
				tag = new VitalRecordTag();
				tag.setVitalRecordId(mtlb.getVitalRecordId());
				tag.setKey(CustomErrorCodes.RV4_MOTHER_FAMILY_TREE_RECORD);
				tag.setContent(mtlb.getMotherId().toString());
				tag.setType(Utils.ERROR_TAG_TYPE);
				tags.add(tag);
			}
		}
		vr.setId(mtlb.getVitalRecordId());
		vr.setFirstname(mtlb.getFirstname());
		vr.setSecondname(mtlb.getSecondname());
		vr.setThirdname(mtlb.getThirdname());
		vr.setFourthname(mtlb.getFourthname());
		vr.setFifthname(mtlb.getFifthname());
		vr.setReportedFatherName(mtlb.getReportedFatherName());
		vr.setReportedMotherName(mtlb.getReportedMotherName());
		vr.setPrimaryCitizenship("191");
		vr.setTypeOfAcquisition(TypeOfAcquisition.BY_DESCENT);
		vr.setTimestamp(mtlb.getTimestamp());
		rvFourBirthMtlbUtils.validateBiographic(mtlb);
		if (!StringUtils.isAllBlank(mtlb.getBirthPlace())) {
			GeoData location = locationsRepository.findByAreaId(mtlb.getBirthPlace());
			if (location != null)
				vr.setBirthPlace(location.getAreaId());
			else {
				VitalRecordTag tag = new VitalRecordTag();
				tag.setVitalRecordId(mtlb.getVitalRecordId());
				tag.setContent(mtlb.getBirthPlace());
				tag.setType(Utils.WARNING_TAG_TYPE);
				tag.setKey(INVALID_BIRTH_PLACE_DATA);
				tags.add(tag);
			}
		}

		vr.setBirthdate(mtlb.getBirthdate());
		vr.setBirthTime(mtlb.getBirthtime());
		vr.setBirthYearUnknown(mtlb.isBirthYearUnknown());
		vr.setBirthMonthUnknown(mtlb.isBirthMonthUnknown());
		vr.setBirthDayUnknown(mtlb.isBirthDayUnknown());
		if (mtlb.getGender() != null)
			vr.setGender(mtlb.getGender().shortValue());
		vr.setMaritalStatus(mtlb.getMaritalStatus());

		Occupation occupation = null;
		if (!StringUtils.isAllBlank(mtlb.getOccupation())) {
			String legacyOccupation = mtlb.getOccupation().trim();
			occupation = new Occupation();
			List<OccupationMetadata> omList = occupationMetadataRepository.findByName(legacyOccupation);
			if (omList.size() > 0)
				occupation.setOccupationId(omList.get(0).getId());
			else {
				OccupationMetadata om = new OccupationMetadata();
				om.setName(legacyOccupation);
				occupationMetadataRepository.insert(om);
				occupation.setOccupationId(om.getId());
			}
			occupation.setVitalRecordId(mtlb.getVitalRecordId());
		}

		Email email = null;
		if (!StringUtils.isAllBlank(mtlb.getEmail())) {
			String legacyEmail = mtlb.getEmail().trim();
			email = new Email();
			email.setEmail(legacyEmail);
			email.setVitalRecordId(mtlb.getVitalRecordId());
		}

		PhoneNumber phoneNumber = null;
		if (!StringUtils.isAllBlank(mtlb.getPhoneNumber())) {
			String legacyPhoneNumber = mtlb.getPhoneNumber().trim();
			phoneNumber = new PhoneNumber();
			phoneNumber.setPhoneNumber(legacyPhoneNumber);
			phoneNumber.setVitalRecordId(mtlb.getVitalRecordId());
		}

		Address address = null;
		if (!StringUtils.isAllBlank(mtlb.getAddress())) {
			VitalRecordTag tag = null;
			GeoData location =locationsRepository.findByAreaId(mtlb.getAddress());
			if (location != null) {
				address = new Address();
				address.setAddressType(AddressType.RESIDENCY);
				address.setAreaId(location.getAreaId());
				address.setVitalRecordId(mtlb.getVitalRecordId());
			}
			else {
				tag = new VitalRecordTag();
				tag.setVitalRecordId(mtlb.getVitalRecordId());
				tag.setContent(mtlb.getBirthPlace());
				tag.setType(Utils.WARNING_TAG_TYPE);
				tag.setKey(INVALID_RESIDENCE_ADDRESS_DATA);
				tags.add(tag);
			}
			if (tag != null)
				tags.add(tag);
		}

		if (mtlb.isMultiBirth()) {
			VitalRecordTag tag = new VitalRecordTag();
			tag.setContent("Multiple Births");
			tag.setType(Utils.INFO_TAG_TYPE);
			tag.setVitalRecordId(mtlb.getVitalRecordId());
			tags.add(tag);
		}

		Long faceId = null;
		if (mtlb.getFace() != null) {
			Face face = new Face();
			face.setImage(mtlb.getFace());
			face.setVitalRecordId(mtlb.getVitalRecordId());
			faceId = faceRepository.insert(face);
		}

		ArchideMtlbAnnex annex = null;
		try {
			annex = ArchideMtlbAnnexUtil.deserialize(mtlb.getAnnex());
		} catch (JsonProcessingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if (annex.getTags() != null)
			for (ArchideMtlbAnnexTag tag : annex.getTags()) {
				VitalRecordTag vt = new VitalRecordTag();
				vt.setVitalRecordId(mtlb.getVitalRecordId());
				vt.setContent(tag.getValue());
				vt.setType(tag.getType());
				vt.setKey(tag.getName());
				tags.add(vt);
			}
		
		
		List<RvfourBirthMtlbTag> mtlbTagList = rvfourBirthMtlbTagRepository.findByRvfourBirthMtlbId(mtlb.getId());
		for (RvfourBirthMtlbTag tag : mtlbTagList) {
			VitalRecordTag vt = new VitalRecordTag();
			vt.setVitalRecordId(mtlb.getVitalRecordId());
			vt.setContent(tag.getContent());
			vt.setType(tag.getType());
			vt.setKey(tag.getKey());
			tags.add(vt);
		}
		ArrayList<VitalRecordNote> notes = new ArrayList<>();
		for (ArchideMtlbAnnexLogEntry note : annex.getNotes()) {
			VitalRecordNote vn = new VitalRecordNote();
			vn.setAddedBy(note.getAuthor());
			vn.setContent(note.getContent());
			vn.setTimestamp(note.getTimestamp());
			vn.setVitalRecordId(mtlb.getVitalRecordId());
			notes.add(vn);
		}

		ArrayList<VitalRecordMention> mentions = new ArrayList<>();
		for (ArchideMtlbAnnexLogEntry mention : annex.getMentions()) {
			VitalRecordMention vm = new VitalRecordMention();
			vm.setAddedBy(mention.getAuthor());
			vm.setContent(mention.getContent());
			vm.setTimestamp(mention.getTimestamp());
			vm.setVitalRecordId(mtlb.getVitalRecordId());
			mentions.add(vm);
		}

		ArrayList<VitalRecordAttachment> attachments = new ArrayList<>();
		List<RvfourBirthMtlbAttachment> mtlbAttachments = rv4BirthMtlbAttachmentRepository
				.findByRvfourBirthMtlbId(mtlb.getId());
		for (RvfourBirthMtlbAttachment mtlbAttachment: mtlbAttachments) {
			VitalRecordAttachment attachment = new VitalRecordAttachment();
			attachment.setVitalRecordId(mtlb.getVitalRecordId());
			attachment.setContent(mtlbAttachment.getContent());
			attachment.setFileName(mtlbAttachment.getFileName());
			attachment.setDescription(mtlbAttachment.getDescription());
			attachment.setPreview(mtlbAttachment.getPreview());
			attachments.add(attachment);
		}

		try {
			vitalRecordRepository.insert(vr);
			if (occupation != null)
				occupationRepository.insert(occupation);
			if (email != null)
				emailRepository.insert(email);
			if (phoneNumber != null)
				phoneNumberRepository.insert(phoneNumber);
			if (address != null)
				addressRepository.insert(address);
			for (VitalRecordTag tag : tags)
				vitalRecordTagRepository.insert(tag);
			if (faceId != null) {
				vitalRecordRepository.setFaceIdById(vr.getId(), faceId);
			}
			for (VitalRecordAttachment attachment : attachments)
				vitalRecordAttachmentRepository.insert(attachment);
			for (VitalRecordNote note : notes)
				vitalRecordNoteRepository.insert(note);
			for (VitalRecordMention mention : mentions)
				vitalRecordMentionRepository.insert(mention);
			mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	
}
