package systems.archide.core.workflows;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import systems.archide.core.pojo.DeathRecordNote;
import systems.archide.core.pojo.DeathRecordTag;
import systems.archide.core.pojo.DocumentType;
import systems.archide.core.pojo.Face;
import systems.archide.core.pojo.GlobalState;
import systems.archide.core.pojo.IssuedDocuments;
import systems.archide.core.pojo.IssuedDocumentsTag;
import systems.archide.core.pojo.MtlbStatus;
import systems.archide.core.pojo.RvfourBirthMtlb;
import systems.archide.core.pojo.RvfourIdCardMtlb;
import systems.archide.core.pojo.VitalRecord;
import systems.archide.core.repository.FaceRepository;
import systems.archide.core.repository.GlobalStateRepository;
import systems.archide.core.repository.IdcoreCustomQueries;
import systems.archide.core.repository.IssuedDocumentsRepository;
import systems.archide.core.repository.IssuedDocumentsTagRepository;
import systems.archide.core.repository.RvfourIdCardMtlbRepository;
import systems.archide.core.repository.VitalRecordRepository;
import systems.archide.core.workflows.utils.CustomErrorCodes;
import systems.archide.core.workflows.utils.Utils;

@Component
public class RvfourIdCardMtlbSubmitted {
	
	private static final Logger log = LoggerFactory.getLogger(RvfourIdCardMtlbSubmitted.class);
	
	@Autowired
	GlobalStateRepository globalStateRepository;
	
	@Autowired
	RvfourIdCardMtlbRepository rvfourIdCardMtlbRepository;
	
	@Autowired
	IssuedDocumentsRepository issuedDocumentsRepository;
	
	@Autowired
	IssuedDocumentsTagRepository issuedDocumentsTagRepository;
	
	@Autowired
	VitalRecordRepository vitalRecordRepository;
	
	@Autowired
	FaceRepository faceRepository;
	
	@Autowired
	IdcoreCustomQueries queries;
	
	@Autowired
	Utils utils;

	@Scheduled(fixedDelay = 5000)
	public void fromSubmittedStatus() {
		GlobalState migrationState = globalStateRepository.findByKey("migration");
		List<RvfourIdCardMtlb> mtlbList = queries.getRandomRv4IdCardMtlbByMtlbStatus(MtlbStatus.SUBMITTED.getValue());
		if (mtlbList.size() > 0) {
			log.info("Read {} newly submitted rv4 ID Card applications", mtlbList.size());
			for (RvfourIdCardMtlb mtlb : mtlbList)	{
				createIssuedDocument(mtlb, migrationState);	
				rvfourIdCardMtlbRepository.save(mtlb);
			}
		}
	}

	private void createIssuedDocument(RvfourIdCardMtlb mtlb, GlobalState migrationState) {
		IssuedDocuments document = new IssuedDocuments();
		ArrayList<IssuedDocumentsTag> tags = new ArrayList<IssuedDocumentsTag>();
		int personRecordStatus = utils.checkVitalRecordStatus(mtlb.getVitalRecordId(), migrationState);	
		if (personRecordStatus == Utils.UNKNOWN || personRecordStatus == Utils.FOUND_BUT_VITAL_RECORD_NOT_CREATED_YET)
			return;
		else if (personRecordStatus == Utils.CONFIRMED_MISSING) {
			IssuedDocumentsTag tag = null;
			tag = new IssuedDocumentsTag();
			tag.setContent(CustomErrorCodes.VITAL_RECORD_ID_MISSING);
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
		}
		else if (personRecordStatus == Utils.FOUND_AS_FAMILY_TREE_RECORD) {
			IssuedDocumentsTag tag = null;
			tag = new IssuedDocumentsTag();
			tag.setKey(CustomErrorCodes.RECORD_FOUND_IN_FAMILY_TREE);
			tag.setContent(mtlb.getVitalRecordId().toString());
			tag.setType(Utils.ERROR_TAG_TYPE);
			tags.add(tag);
		}
		if (personRecordStatus == Utils.FOUND) {
			List<Face> faceList = faceRepository.findByVitalRecordId(mtlb.getVitalRecordId());
			VitalRecord vr = vitalRecordRepository.findById(mtlb.getVitalRecordId());
			if (!faceList.isEmpty())
				document.setFace(faceList.get(faceList.size() - 1).getImage());
			else {
				IssuedDocumentsTag tag = null;
				tag = new IssuedDocumentsTag();
				tag.setContent(CustomErrorCodes.NO_PICTURE_FOUND);
				tag.setType(Utils.ERROR_TAG_TYPE);
				tags.add(tag);
			}
			String fullName="";
			if (!StringUtils.isAllBlank(vr.getFirstname()))
				fullName = vr.getFirstname();
			if (!StringUtils.isAllBlank(vr.getSecondname()))
				fullName = fullName.concat(" " + vr.getSecondname());
			if (!StringUtils.isAllBlank(vr.getThirdname()))
				fullName = fullName.concat(" " + vr.getThirdname());
			if (!StringUtils.isAllBlank(vr.getFourthname()))
				fullName = fullName.concat(" " + vr.getFourthname());
			document.setFullName(fullName);
		}
		document.setCancelledDate(mtlb.getCancelledDate());
		document.setCancelledReason(mtlb.getCancelledReason());
		document.setDocumentId(mtlb.getIdCardNumber());
		document.setDocumentType(DocumentType.ID_CARD);
		document.setExpires(mtlb.getExpires());
		document.setIssuedDate(mtlb.getIssuedDate());
		document.setVitalRecordId(mtlb.getVitalRecordId());
		
		try {
			Long id = issuedDocumentsRepository.insert(document);
			for (IssuedDocumentsTag tag : tags) {
				tag.setIssuedDocumentsId(id);
				issuedDocumentsTagRepository.insert(tag);
			}
			if (personRecordStatus == Utils.FOUND) {
				VitalRecord vitalRecord = vitalRecordRepository.findById(mtlb.getVitalRecordId());
				vitalRecord.setCardId(id);
				vitalRecordRepository.save(vitalRecord);
			}
			mtlb.setMtlbStatus(MtlbStatus.ARCHIVED);

		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}
