package systems.archide.core.workflows.utils;


import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import systems.archide.core.ErrorCodes;
import systems.archide.core.pojo.*;
import systems.archide.core.repository.*;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.time.LocalDateTime;
import java.util.List;

@Component
public class CivilStatusMtlbUtils {

	final String ERROR_TAG_TYPE = "ERROR";



	@Autowired
	IdcoreCustomQueries customQueries;
	@Autowired
	CivilStatusMtlbRepository civilStatusMtlbRepository;
	@Autowired
	CivilStatusMtlbTagRepository civilStatusMtlbTagRepository;

	@Autowired
	VitalRecordRepository vitalRecordRepository;

	@Autowired
	CivilStatusMtlbAttachmentRepository civilStatusMtlbAttachmentRepository;
	
	@Autowired
	VitalRecordMedicalNotificationFileRepository vitalRecordMedicalNotificationFileRepository;
	
	@Autowired
	CivilStatusMtlbMedicalNotificationFileRepository civilStatusMtlbMedicalNotificationFileRepository;

	@Autowired
	AddressRepository addressRepository;

	@Autowired
	EmailRepository emailRepository;

	@Autowired
	PhoneNumberRepository phoneNumberRepository;

	@Autowired
	FaceRepository faceRepository;

	@Autowired
	VitalRecordAttachmentRepository vitalRecordAttachmentRepository;

	@Autowired
	DigitalizationQueueRepository digitalizationQueueRepository;
	
	@Autowired
	CivilStatusMtlbNoteRepository civilStatusMtlbNoteRepository;
	
	@Autowired
	VitalRecordNoteRepository vitalRecordNoteRepository;
	
	@Autowired
	VitalRecordHistoryRepository vitalRecordHistoryRepository;
	
	@Autowired
	VitalRecordTagRepository vitalRecordTagRepository;
	
	@Autowired
	VitalRecordTagHistoryRepository vitalRecordTagHistoryRepository;

	public boolean mtlbWithoutFingerprints(CivilStatusMtlb mtlb) {
		return mtlb.getRightThumb() == null && mtlb.getRightIndexFinger() == null && mtlb.getRightMiddleFinger() == null
				&& mtlb.getRightRingFinger() == null && mtlb.getRightPinky() == null && mtlb.getLeftThumb() == null
				&& mtlb.getLeftIndexFinger() == null && mtlb.getLeftMiddleFinger() == null
				&& mtlb.getLeftRingFinger() == null && mtlb.getLeftPinky() == null;
	}

	public boolean validateBiographic(CivilStatusMtlb mtlb) {
		boolean validName = validateName(mtlb);
		boolean validGenderAndBirthDate = validateGenderAndBirthDate(mtlb);
		boolean validAddress = validateAddress(mtlb);
		boolean validMedicalNotification = validateMedicalNotification(mtlb);
		boolean validFatherAndMother = validateFatherAndMother(mtlb);
		return validName && validGenderAndBirthDate && validAddress && validMedicalNotification && validFatherAndMother;
	}

	private boolean validateFatherAndMother(CivilStatusMtlb mtlb) {
		boolean valid = true;
		if (mtlb.getMotherId() != null
				|| (mtlb.getReportedMotherName() != null && !mtlb.getReportedMotherName().isEmpty())) {
			if (mtlb.getMotherId() != null) {
				VitalRecord motherRecord = vitalRecordRepository.findById(mtlb.getMotherId());
				if (motherRecord == null) {
					createErrorTag(ErrorCodes.MOTHER_NOT_FOUND, mtlb.getId());
					valid = false;
				}
			}
		} else if (!mtlb.isUnknownMother()) {
			createErrorTag(ErrorCodes.MOTHER_NOT_FOUND, mtlb.getId());
			valid = false;
		}
		if (mtlb.getFatherId() != null
				|| (mtlb.getReportedFatherName() != null && !mtlb.getReportedFatherName().isEmpty())) {
			if (mtlb.getFatherId() != null) {
				VitalRecord fatherRecord = vitalRecordRepository.findById(mtlb.getFatherId());
				if (fatherRecord == null) {
					createErrorTag(ErrorCodes.FATHER_NOT_FOUND, mtlb.getId());
					valid = false;
				}
			}
		} else if (!mtlb.isUnknownFather()) {
			createErrorTag(ErrorCodes.FATHER_NOT_FOUND, mtlb.getId());
			valid = false;
		}

		return valid;
	}

	private boolean validateMedicalNotification(CivilStatusMtlb mtlb) {
		if (!mtlb.isMedicalNotification() || mtlb.isMedicalNotification() == null) {
			if (mtlb.getWitness1Id() == null || mtlb.getWitness2Id() == null) {
				createErrorTag(ErrorCodes.WITNESSES_MISSING, mtlb.getId());
				return false;
			}
		} else {
			boolean attachmentExists = checkAttachments(mtlb);
			if (!attachmentExists)
				if (mtlb.getWitness1Id() == null || mtlb.getWitness2Id() == null) {
					createErrorTag(ErrorCodes.WITNESSES_MISSING, mtlb.getId());
					return false;
				}
		}
		return true;
	}

	private boolean checkAttachments(CivilStatusMtlb mtlb) {
		List<CivilStatusMtlbMedicalNotificationFile> attachmentList = civilStatusMtlbMedicalNotificationFileRepository
				.findByCivilStatusMtlbId(mtlb.getId());
		if (attachmentList.isEmpty())
			return false;
		return true;
	}

	private boolean validateAddress(CivilStatusMtlb mtlb) {
		List<Address> addressList = addressRepository.findByCivilStatusMtlbId(mtlb.getId());
		if (addressList.isEmpty()) {
			createErrorTag(ErrorCodes.ADDRESS_MISSING, mtlb.getId());
			return false;
		}
		return true;
	}

	private boolean validateGenderAndBirthDate(CivilStatusMtlb mtlb) {
		boolean valid = true;
		if (mtlb.getGender() == null) {
			createErrorTag(ErrorCodes.GENDER_MISSING, mtlb.getId());
			valid = false;
		}
		if (mtlb.getBirthdate() == null) {
			createErrorTag(ErrorCodes.BIRTH_DATE_MISSING, mtlb.getId());
			valid = false;
		}
		return valid;
	}

	private boolean validateName(CivilStatusMtlb mtlb) {
		boolean valid = true;
		if (StringUtils.isAllBlank(mtlb.getFirstname())) {
			createErrorTag(ErrorCodes.FIRST_NAME_MISSING, mtlb.getId());
			valid = false;
		}
		if (StringUtils.isAllBlank(mtlb.getFourthname())) {
			createErrorTag(ErrorCodes.FAMILY_NAME_MISSING, mtlb.getId());
			valid = false;
		}
		String match = "^[A-Z�-�][A-Za-z�-�]{2,14}$";
		String middleNameMatch = "^[A-Z�-�][A-Za-z�-�]{2,28}$";
		if (mtlb.getFirstname() != null)
			if (!mtlb.getFirstname().matches(match)) {
				createErrorTag(ErrorCodes.FIRST_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		if (mtlb.getSecondname() != null)
			if (!mtlb.getSecondname().matches(middleNameMatch)) {
				createErrorTag(ErrorCodes.SECOND_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		if (mtlb.getThirdname() != null)
			if (!mtlb.getThirdname().matches(match)) {
				createErrorTag(ErrorCodes.THIRD_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		if (mtlb.getFourthname() != null)
			if (!mtlb.getFourthname().matches(match)) {
				createErrorTag(ErrorCodes.FAMILY_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		if (mtlb.getFifthname() != null)
			if (!mtlb.getFifthname().matches(match)) {
				createErrorTag(ErrorCodes.MARRIAGE_NAME_CHAR_RULES, mtlb.getId());
				valid = false;
			}
		return valid;
	}

	public void createErrorTag(String errorCode, Long mtlbId) {
		CivilStatusMtlbTag tag = new CivilStatusMtlbTag();
		tag.setContent(errorCode);
		tag.setType(ERROR_TAG_TYPE);
		tag.setCivilStatusMtlbId(mtlbId);
		civilStatusMtlbTagRepository.insert(tag);
	}

	public void createErrorTag(String errorCode, Long mtlbId, String type) {
		CivilStatusMtlbTag tag = new CivilStatusMtlbTag();
		tag.setContent(errorCode);
		tag.setType(type);
		tag.setCivilStatusMtlbId(mtlbId);
		civilStatusMtlbTagRepository.insert(tag);
	}

	public boolean validateNoExistingCivilStatusMtlbFirstNameAndFourthName(CivilStatusMtlb mtlb) {
		List<CivilStatusMtlb> civilMtlIdList = customQueries
				.findCivilStatusMtlbByFirstnameAndFourthname(mtlb.getFirstname(), mtlb.getFourthname(), mtlb.getId());
		if (civilMtlIdList.isEmpty())
			return true;
		else
			createErrorTag(ErrorCodes.BIRTH_APPLICATION_FIRSTNAME_FOURTHNAME_DUPLICATED, mtlb.getId());
		return true;
	}

	public boolean validateNoExistingVitalRecordFirstNameAndFourthName(CivilStatusMtlb mtlb) {
		List<VitalRecord> vitalRecordIdList = customQueries.findVitalRecordByFirstnameAndFourthname(mtlb.getFirstname(),
				mtlb.getFourthname(), mtlb.getId());
		if (vitalRecordIdList.isEmpty())
			return true;
		else
			createErrorTag(ErrorCodes.BIRTH_VITALRECORD_FIRSTNAME_FOURTHNAME_DUPLICATED, mtlb.getId());
		return true;
	}

	public boolean validateNoExistingCivilStatusMtlbOrVitalRecordFirstNameAndFourthName(CivilStatusMtlb mtlb) {
		return validateNoExistingCivilStatusMtlbFirstNameAndFourthName(mtlb)
				&& validateNoExistingVitalRecordFirstNameAndFourthName(mtlb);
	}

	public boolean validateNoExistingSimilarBReq(CivilStatusMtlb mtlb) {
		List<CivilStatusMtlb> civilMtlIdList = customQueries.findCivilStatusMtlbByBirthdateAndMotherIdAndType(
				mtlb.getMotherId(), mtlb.getMtlbType().getValue(), mtlb.getBirthdate(), mtlb.getId());
		List<VitalRecord> vitalRecordIdList = customQueries.findVitalRecordByBirthdateAndMotherId(mtlb.getMotherId(),
				mtlb.getBirthdate(), mtlb.getId());
		if (civilMtlIdList.isEmpty() && vitalRecordIdList.isEmpty())
			return true;
		if (!civilMtlIdList.isEmpty())
			createErrorTag(ErrorCodes.BIRTH_APPLICATION_DUPLICATED, mtlb.getId());
		if (!vitalRecordIdList.isEmpty())
			createErrorTag(ErrorCodes.BIRTH_VITALRECORD_DUPLICATED, mtlb.getId());
		return true;
	}

	public Long saveFace(byte[] image, Long vitalRecordId) {
		Face face = new Face();
		face.setImage(image);
		face.setVitalRecordId(vitalRecordId);
		return faceRepository.insert(face);
	}

	public Long createVitalRecordFromCivilStatusMtlb(CivilStatusMtlb mtlb) {
		VitalRecord vitalRecord = new VitalRecord();
		mtlbToVitalRecord(mtlb, vitalRecord);
		return vitalRecordRepository.insert(vitalRecord);
	}

	public void addFace2VitalRecord(Long faceId, Long vitalRecordId) {
		VitalRecord vitalRecord = vitalRecordRepository.findById(vitalRecordId);
		vitalRecord.setFaceId(faceId);
		vitalRecordRepository.update(vitalRecord);
	}

	public void saveAttachments(Long mtlbId, Long vitalRecordId) {
		List<CivilStatusMtlbAttachment> civilStatusMtlbAttachments = civilStatusMtlbAttachmentRepository
				.findByCivilStatusMtlbId(mtlbId);		
		for (CivilStatusMtlbAttachment civilStatusMtlbAttachment : civilStatusMtlbAttachments) {
			VitalRecordAttachment vitalRecordAttachment = new VitalRecordAttachment();
			vitalRecordAttachment.setFileName(civilStatusMtlbAttachment.getFileName());
			vitalRecordAttachment.setContent(civilStatusMtlbAttachment.getContent());
			vitalRecordAttachment.setVitalRecordId(vitalRecordId);
			vitalRecordAttachment.setPreview(civilStatusMtlbAttachment.getPreview());
			vitalRecordAttachment.setDescription(civilStatusMtlbAttachment.getDescription());
			vitalRecordAttachmentRepository.insert(vitalRecordAttachment);
		}
		List<CivilStatusMtlbMedicalNotificationFile> medicalNotifList = civilStatusMtlbMedicalNotificationFileRepository
				.findByCivilStatusMtlbId(mtlbId);
		for (CivilStatusMtlbMedicalNotificationFile m : medicalNotifList) {
			VitalRecordMedicalNotificationFile vitalRecordMedicalNotif = new VitalRecordMedicalNotificationFile();
			vitalRecordMedicalNotif.setFileName(m.getFileName());
			vitalRecordMedicalNotif.setContent(m.getContent());
			vitalRecordMedicalNotif.setVitalRecordId(vitalRecordId);
			vitalRecordMedicalNotif.setDescription(m.getDescription());
			vitalRecordMedicalNotif.setPreview(m.getPreview());
			vitalRecordMedicalNotificationFileRepository.insert(vitalRecordMedicalNotif);
		}
	}

	public void saveVitalRecordAdminInfo(Long civilStatusMtlbId, Long vitalRecordId) {
		List<Address> addressList = addressRepository.findByCivilStatusMtlbId(civilStatusMtlbId);
		List<Email> emailList = emailRepository.findByCivilStatusMtlbId(civilStatusMtlbId);
		List<PhoneNumber> phoneNumberList = phoneNumberRepository.findByCivilStatusMtlbId(civilStatusMtlbId);
		for (Address address : addressList) {
			address.setVitalRecordId(vitalRecordId);
			addressRepository.update(address);
		}
		for (Email email : emailList) {
			email.setVitalRecordId(vitalRecordId);
			emailRepository.update(email);
		}
		for (PhoneNumber phone : phoneNumberList) {
			phone.setVitalRecordId(vitalRecordId);
			phoneNumberRepository.update(phone);
		}
	}

	public Long saveDeathRecordFace(byte[] image, Long deathRecordId) {
		Face face = new Face();
		face.setImage(image);
		face.setDeathRecordId(deathRecordId);
		return faceRepository.insert(face);
	}


	public void deleteDigitalizationFromQueue(Long mtlbId) {
		List<CivilStatusMtlbAttachment> civilStatusMtlbAttachments = civilStatusMtlbAttachmentRepository
				.findByCivilStatusMtlbId(mtlbId);
		for (CivilStatusMtlbAttachment civilStatusMtlbAttachment : civilStatusMtlbAttachments) {
			List<DigitalizationQueue> digitalizationQueues = digitalizationQueueRepository
					.findByFileName(civilStatusMtlbAttachment.getFileName());
			for (DigitalizationQueue digitalizationQueue : digitalizationQueues)
				digitalizationQueueRepository.deleteById(digitalizationQueue.getId());
		}
	}
	
	public void mtlbToVitalRecord(CivilStatusMtlb mtlb, VitalRecord vitalRecord) {
		vitalRecord.setFirstname(mtlb.getFirstname());
		vitalRecord.setSecondname(mtlb.getSecondname());
		vitalRecord.setThirdname(mtlb.getThirdname());
		vitalRecord.setFourthname(mtlb.getFourthname());
		vitalRecord.setFifthname(mtlb.getFifthname());
		vitalRecord.setBirthdate(mtlb.getBirthdate());
		vitalRecord.setBirthPlace(mtlb.getBirthPlace());
		vitalRecord.setBirthCountry(mtlb.getBirthCountry());
		vitalRecord.setBirthDayUnknown(mtlb.isBirthDayUnknown());
		vitalRecord.setBirthMonthUnknown(mtlb.isBirthMonthUnknown());
		vitalRecord.setBirthYearUnknown(mtlb.isBirthYearUnknown());
		vitalRecord.setGender(mtlb.getGender());
		vitalRecord.setDisability(mtlb.isDisability());
		vitalRecord.setMaritalStatus(mtlb.getMaritalStatus());
		vitalRecord.setNotes(mtlb.getNotes());
		vitalRecord.setPrimaryCitizenship(mtlb.getPrimaryCitizenship());
		vitalRecord.setOtherCitizenshipCsv(mtlb.getOtherCitizenshipCsv());
		vitalRecord.setFatherId(mtlb.getFatherId());
		vitalRecord.setMotherId(mtlb.getMotherId());
		vitalRecord.setReportedFatherName(mtlb.getReportedFatherName());
		vitalRecord.setReportedMotherName(mtlb.getReportedMotherName());
		vitalRecord.setInactive(false);
		vitalRecord.setTypeOfAcquisition(mtlb.getTypeOfAcquisition());
		vitalRecord.setVillageOfOrigin(mtlb.getVillageOfOrigin());
		vitalRecord.setUnknownFather(mtlb.isUnknownFather());
		vitalRecord.setUnknownMother(mtlb.isUnknownMother());
		vitalRecord.setBirthTime(mtlb.getBirthTime());
		vitalRecord.setTimestamp(LocalDateTime.now());
		vitalRecord.setAdoptiveFatherId(mtlb.getAdoptiveFatherId());
		vitalRecord.setAdoptiveMotherId(mtlb.getAdoptiveMotherId());
		vitalRecord.setMarriageCertificateNumberAdoptive(mtlb.getMarriageCertificateNumberAdoptive());
		vitalRecord.setCourtAdoptionNumber(mtlb.getCourtAdoptionNumber());
		vitalRecord.setVillageOfOriginAdoptive(mtlb.getVillageOfOriginAdoptive());
		vitalRecord.setDeclarantId(mtlb.getDeclarantId());
		vitalRecord.setWitness1Id(mtlb.getWitness1Id());
		vitalRecord.setWitness2Id(mtlb.getWitness2Id());
		vitalRecord.setRegistrationLocation(mtlb.getRegistrationLocation());
		vitalRecord.setMentions(mtlb.getMentions());
	}

	public void updateVitalRecordFromCivilStatusMtlb(CivilStatusMtlb mtlb, VitalRecord vitalRecord) {
		saveHistory(vitalRecord);
		mtlbToVitalRecord(mtlb, vitalRecord);
		vitalRecordRepository.update(vitalRecord);
	}
	
	private void saveHistory(VitalRecord vitalRecord) {
		VitalRecordHistory history = new VitalRecordHistory();
		history.setFirstname(vitalRecord.getFirstname());
		history.setFifthname(vitalRecord.getFifthname());
		history.setSecondname(vitalRecord.getSecondname());
		history.setThirdname(vitalRecord.getThirdname());
		history.setFourthname(vitalRecord.getFourthname());
		history.setBirthdate(vitalRecord.getBirthdate());
		history.setBirthPlace(vitalRecord.getBirthPlace());
		history.setBirthCountry(vitalRecord.getBirthCountry());
		history.setBirthDayUnknown(vitalRecord.isBirthDayUnknown());
		history.setBirthMonthUnknown(vitalRecord.isBirthMonthUnknown());
		history.setBirthYearUnknown(vitalRecord.isBirthYearUnknown());
		history.setGender(vitalRecord.getGender());
		history.setDisability(vitalRecord.isDisability());
		history.setMaritalStatus(vitalRecord.getMaritalStatus());
		history.setNotes(vitalRecord.getNotes());
		history.setPrimaryCitizenship(vitalRecord.getPrimaryCitizenship());
		history.setOtherCitizenshipCsv(vitalRecord.getOtherCitizenshipCsv());
		history.setFatherId(vitalRecord.getFatherId());
		history.setMotherId(vitalRecord.getMotherId());
		history.setReportedFatherName(vitalRecord.getReportedFatherName());
		history.setReportedMotherName(vitalRecord.getReportedMotherName());
		history.setInactive(false);
		history.setHistoryTimestamp(LocalDateTime.now());
		history.setTimestamp(vitalRecord.getTimestamp());
		history.setId(vitalRecord.getId());
		history.setAdoptiveFatherId(vitalRecord.getAdoptiveFatherId());
		history.setAdoptiveMotherId(vitalRecord.getAdoptiveMotherId());
		history.setVillageOfOriginAdoptive(vitalRecord.getVillageOfOriginAdoptive());
		history.setMarriageCertificateNumberAdoptive(vitalRecord.getMarriageCertificateNumberAdoptive());
		history.setCourtAdoptionNumber(vitalRecord.getCourtAdoptionNumber());
		history.setDeclarantId(vitalRecord.getDeclarantId());
		history.setWitness1Id(vitalRecord.getWitness1Id());
		history.setWitness2Id(vitalRecord.getWitness2Id());
		vitalRecordHistoryRepository.insert(history);
	}

	public void saveNotes(Long id, Long vitalRecordId) {
		List<CivilStatusMtlbNote> cnoteList = civilStatusMtlbNoteRepository.findByCivilStatusMtlbId(id);
		for (CivilStatusMtlbNote cnote : cnoteList) {
			VitalRecordNote vnote = new VitalRecordNote();
			vnote.setAddedBy(cnote.getAddedBy());
			vnote.setContent(cnote.getContent());
			vnote.setTimestamp(cnote.getTimestamp());
			vnote.setVitalRecordId(vitalRecordId);
			vitalRecordNoteRepository.insert(vnote);
		}
		
	}
	
	public void deleteRecordAttachments(Long vitalRecordId) {
		List<VitalRecordAttachment> attachmentList = vitalRecordAttachmentRepository.findByVitalRecordId(vitalRecordId); 
		List<VitalRecordMedicalNotificationFile> medicalNotificationFileList = vitalRecordMedicalNotificationFileRepository.findByVitalRecordId(vitalRecordId); 
		for (VitalRecordAttachment a : attachmentList)
			vitalRecordAttachmentRepository.purge(a);
		for (VitalRecordMedicalNotificationFile f : medicalNotificationFileList)
			vitalRecordMedicalNotificationFileRepository.delete(f);
	}

	public void saveVitalRecordHistoryTags(Long id) {
		List<VitalRecordTag> recordTags = vitalRecordTagRepository.findByVitalRecordId(id);
		if (!recordTags.isEmpty()) {
			for (VitalRecordTag tag : recordTags) {
				VitalRecordTagHistory htag = new VitalRecordTagHistory();
				htag.setContent(tag.getContent());
				htag.setKey(tag.getKey());
				htag.setId(tag.getId());
				htag.setType(tag.getType());
				htag.setVitalRecordId(tag.getVitalRecordId());
				vitalRecordTagHistoryRepository.insert(htag);
				vitalRecordTagRepository.purgeById(tag.getId());
			}
		}
		
	}
}
